
local GUI_ID_MenuJournal = tes3ui.registerID("MenuJournal")
local GUI_ID_MenuJournal_bookmark_layout = tes3ui.registerID("MenuJournal_bookmark_layout")
local GUI_ID_MenuJournal_button_bookmark_quests = tes3ui.registerID("MenuJournal_button_bookmark_quests")
local GUI_ID_MenuJournal_button_bookmark_topics = tes3ui.registerID("MenuJournal_button_bookmark_topics")

local common = require("UI Expansion.common")

local function dialogueHasEntry(dialogue)
	for info in tes3.iterate(dialogue.info) do
		if (info.firstHeardFrom) then
			return true
		end
	end
	return false
end

local topicScrollPaneContentPane = nil

local function onFilterChanged()
	for _, element in pairs(topicScrollPaneContentPane.children) do
		element.visible = common.journalTopicFilter:triggerFilter({ text = element.text })
	end
end

local journalTopicFilters = common.createFilterInterface({
	createSearchBar = true,
	createIcons = false,
	createButtons = false,
	searchTextColor = tes3ui.getPalette("journal_topic_color"),
	searchTextPlaceholderColor = tes3ui.getPalette("journal_topic_color"),
	onFilterChanged = onFilterChanged,
})
common.journalTopicFilter = journalTopicFilters

local function createTopicList(e)
	-- Destroy current tab-thing contents.
	local journalMenu = tes3ui.findMenu(GUI_ID_MenuJournal)
	local container = journalMenu:findChild(GUI_ID_MenuJournal_bookmark_layout)
	container.flowDirection = "top_to_bottom"
	container:destroyChildren()

	-- Create a block for our filters.
	local filterBlock = container:createBlock({})
	filterBlock.flowDirection = "left_to_right"
	filterBlock.widthProportional = 1.0
	filterBlock.autoHeight = true
	journalTopicFilters:createElements(filterBlock)

	-- Change 

	-- Create a complete topic list.
	local topicScrollPane = container:createVerticalScrollPane({ hideFrame = true })
	topicScrollPaneContentPane = topicScrollPane.widget.contentPane

	-- Collect a list of valid dialogue entries.
	for dialogue in tes3.iterate(tes3.mobilePlayer.dialogueList) do
		if (dialogueHasEntry(dialogue)) then
			local textSelect = topicScrollPaneContentPane:createTextSelect({ text = dialogue.id })
			textSelect:setPropertyObject("PartHyperText_dialog", dialogue)
			textSelect:register("mouseClick", 0x5D70C0)
	
			local widget = textSelect.widget
			widget.idle = tes3ui.getPalette("journal_topic_color")
			widget.over = tes3ui.getPalette("journal_topic_over_color")
			textSelect.color = tes3ui.getPalette("journal_topic_color")
		end
	end

	journalTopicFilters:focusSearchBar()

	journalMenu:updateLayout()
end

local function createQuestList(e)
	-- Create the list as normal.
	e.source:forwardEvent(e)

	-- Remove the active/inactive filter buttons.


	-- Create a search thing.


	-- Create our own filter buttons.

end

local function onMenuJournalActivated(e)
	if (not e.newlyCreated) then
		return
	end

	local topicsButton = e.element:findChild(GUI_ID_MenuJournal_button_bookmark_topics)
	topicsButton:register("mouseClick", createTopicList)

	local questButton = e.element:findChild(GUI_ID_MenuJournal_button_bookmark_quests)
	questButton:register("mouseClick", createQuestList)
end
event.register("uiActivated", onMenuJournalActivated, { filter = "MenuJournal" })

local function DEBUG_addEntries(e)
	for dialogue in tes3.iterate(tes3.dataHandler.nonDynamicData.dialogues) do
		if (dialogue.type == 0) then
			mwscript.addTopic({ topic = dialogue })
		elseif (dialogue.type == 4) then
			dialogue:addToJournal({ index = 0 })
		end
	end
end
event.register("loaded", DEBUG_addEntries)
